#!/usr/bin/env python
# da_initexit.py

"""
.. module:: initexit
.. moduleauthor:: Wouter Peters 

Revision History:
File created on 13 May 2009.

The CycleControl class is found in the module :mod:`initexit`. It is derived from the standard python :class:`dictionary` object. It is the only core object of CTDAS that is automatically created in the pipeline, the user (normally) does not need to modify or extend it. The class is created based on options and arguments passes on the command line when submitting your main CTDAS job. 

Valid options are defined in 

.. autofunction:: da.tools.initexit.parse_options

With the name of a valid ``rc-file``, the CycleControl object is instantiated and validated. An example rc-file looks
like this:::

    ! Info on the data assimilation cycle

    time.restart        : False                     ! Restart from an existing run T/F
    time.start          : 2000-01-01 00:00:00       ! Start time of first cycle
    time.finish         : 2000-01-08 00:00:00       ! End time of last cycle
    time.cycle          : 7                         ! length of each cycle, 7 means one week
    time.nlag           : 5                         ! number of cycles in one smoother window
    dir.da_run          : ${HOME}/tmp/test_da       ! the run directory for you project

    ! Info on the DA system used

    da.system           : CarbonTracker             ! an identifier for your inversion system
    da.system.rc        : da/rc/carbontracker.rc    ! the settings needed in your inversion system

    ! Info on the forward model to be used

    da.obsoperator         : TM5                                ! an identifier for your observation operator
    da.obsoperator.rc      : ${HOME}/Modeling/TM5/tm5-ctdas.rc  ! the rc-file needed to run youobservation operator
    da.optimizer.nmembers  : 30                                 ! the number of ensemble members desired in the optimization

The most important method of the CycleControl object are listed below:

.. autoclass:: da.tools.initexit.CycleControl 
   :members: initialize, finalize,  collect_restart_data, move_restart_data, 
             submit_next_cycle, setup_file_structure, recover_run, random_seed

Two important attributes of the CycleControl object are:
    (1) DaSystem, an instance of a :ref:`dasystem`
    (2) DaPlatForm, an instance of a :ref:`platform`

Other functions in the module initexit that are related to the control of a DA cycle are:

.. autofunction:: da.tools.initexit.start_logger 
.. autofunction:: da.tools.initexit.validate_opts_args 


"""
import logging
import os
import sys
import shutil

needed_da_items = [
    'time.start',
    'time.finish',
    'time.nlag',
    'time.cycle',
    'dir.da_run',
    'da.system',
    'da.system.rc',
    'da.obsoperator',
    'da.obsoperator.rc',
    'da.optimizer.nmembers']

# only needed in an earlier implemented where each substep was a separate job
# validprocesses = ['start','done','samplestate','advance','invert']


class CycleControl(dict):
    """
    This object controls the CTDAS system flow and functionality.
    """
        
    def __init__(self, opts=[], args={}):
        """
        The CycleControl object is instantiated with a set of options and arguments.
        The list of arguments must contain the name of an existing ``rc-file``. 
        This rc-file is loaded by method :meth:`~da.tools.initexit.CycleControl.load_rc` and validated
        by :meth:`~da.tools.initexit.CycleControl.validate_rc`

        Options for the CycleControl consist of accepted command line flags or arguments 
        in :func:`~da.tools.initexit.CycleControl.parse_options`

        """

        self.load_rc(args['rc'])
        self.validate_rc()
        self.opts = opts

        # Add some useful variables to the rc-file dictionary

        self['jobrcfilename'] = self.RcFileName
        self['dir.da_submit'] = os.getcwd()
        self['da.crash.recover'] = '-r' in opts
        self['verbose'] = '-v' in opts
        self.DaSystem = None # to be filled later
        self.RestartFileList = [] # List of files needed for restart, to be extended later
        self.OutputFileList = [] # List of files needed for output, to be extended later


    def load_rc(self, RcFileName):
        """ 
        This method loads a DA Cycle rc-file with settings for this simulation 
        """
        import da.tools.rc as rc

        rcdata = rc.read(RcFileName)
        for k, v in rcdata.iteritems():
            self[k] = v
        self.RcFileName = RcFileName
        self.DaRcLoaded = True

        logging.info('DA Cycle rc-file (%s) loaded successfully' % self.RcFileName)


    def validate_rc(self):
        """ 
        Validate the contents of the rc-file given a dictionary of required keys. 
        Currently required keys are :attr:`~da.tools.initexit.needed_da_items`
        """
        from da.tools.general import to_datetime

        for k, v in self.iteritems():
            if v in ['True', 'true', 't', 'T', 'y', 'yes']:
                self[k] = True
            if v in ['False', 'false', 'f', 'F', 'n', 'no']:
                self[k] = False
            if 'date' in k : 
                self[k] = to_datetime(v)
            if 'time.start' in k : 
                self[k] = to_datetime(v)
            if 'time.end' in k : 
                self[k] = to_datetime(v)
            if 'time.finish' in k : 
                self[k] = to_datetime(v)
        for key in needed_da_items:
            if not self.has_key(key):
                msg = 'Missing a required value in rc-file : %s' % key
                logging.error(msg)
                logging.error('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ')
                logging.error('Please note the update on Dec 02 2011 where rc-file names for DaSystem and ')
                logging.error('are from now on specified in the main rc-file (see da/rc/da.rc for example)')
                logging.error('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ')
                raise IOError, msg
        logging.debug('DA Cycle settings have been validated succesfully')

    def parse_times(self):
        """ 
        Parse time related parameters into datetime objects for later use 
        """
        from da.tools.general import advance_time

        startdate = self['time.start']
        finaldate = self['time.finish']                  

        if finaldate <= startdate:
            logging.error('The start date (%s) is not greater than the end date (%s), please revise' % (startdate.strftime('%Y%m%d'), finaldate.strftime('%Y%m%d'))) 
            raise ValueError
        cyclelength = self['time.cycle']                 # get time step

# Determine end date

        if cyclelength == 'infinite':
            enddate = finaldate
        else:
            enddate = advance_time(startdate, cyclelength)

        dt = enddate - startdate

        if enddate > finaldate:  # do not run beyond finaldate
            enddate = finaldate

        self['time.start'] = startdate
        self['time.end'] = enddate
        self['time.finish'] = finaldate
        self['cyclelength'] = dt

        logging.info("===============================================================")
        logging.info("DA Cycle start date is %s" % startdate.strftime('%Y-%m-%d %H:%M'))
        logging.info("DA Cycle end date is %s" % enddate.strftime('%Y-%m-%d %H:%M'))
        logging.info("DA Cycle final date is %s" % finaldate.strftime('%Y-%m-%d %H:%M'))  
        logging.info("DA Cycle cycle length is %s" % cyclelength)
        logging.info("DA Cycle restart is %s" % str(self['time.restart']))
        logging.info("===============================================================")


    def set_sample_times(self, lag):
        """
        Set the times over which a sampling interval will loop, depending on 
        the lag. Note that lag falls in the interval [0,nlag-1]
        """
        import copy

        # Start from cycle times 
        self['time.sample.start'] = copy.deepcopy(self['time.start'])
        self['time.sample.end'] = copy.deepcopy(self['time.end'])

        # Now advance depending on lag

        for l in range(lag):
            self.advance_sample_times()


    def advance_sample_times(self):
        """ 
        Advance sampling start and end time by one cycle interval
        """
        from da.tools.general import advance_time

        days = self['cyclelength'].days                

        self['time.sample.start'] = advance_time(self['time.sample.start'], days)
        self['time.sample.end'] = advance_time(self['time.sample.end'], days)
    

    def advance_cycle_times(self):
        """ 
        Advance cycle start and end time by one cycle interval
        """
        from da.tools.general import advance_time
              
        days = self['cyclelength'].days                  

        startdate = advance_time(self['time.start'], days)
        enddate = advance_time(self['time.end'], days)

        filtertime = startdate.strftime('%Y%m%d')
        self['dir.output'] = os.path.join(self['dir.da_run'], 'output', filtertime)

        self['time.start'] = startdate
        self['time.end'] = enddate


    def random_seed(self, action='read'):
        """ 
        Get the randomseed and save it, or read the random seed and set it. The seed is currently stored
        in a python :mod:`pickle` file, residing in the ``exec`` directory

        """
        import cPickle
        import numpy as np

        filename = os.path.join(self['dir.exec'], 'randomseed.pickle')

        if action == 'write':
            f = open(filename, 'wb')
            seed = np.random.get_state()
            cPickle.dump(seed, f, -1)
            f.close()

            msg = "Saved the random seed generator values to file"

        if action == 'read':
            f = open(filename, 'rb')
            seed = cPickle.load(f)
            np.random.set_state(seed)
            f.close()

            msg = "Retrieved the random seed generator values from file"

        logging.info(msg)
        self.RestartFileList.append(filename)
        logging.debug("Added the randomseed.pickle file to the RestartFileList")


    def initialize(self):
        """ 
        This method determines how to proceed with the cycle. Three options are implemented:

            1. *Fresh start*  : set up the required file structure for this simulation and start
            2. *Restart*      : use latest da_runtime variables from the exec dir and restart
            3. *Recover*      : restart after crash by getting data from restart/one-ago folder

        The choice that gets executed depends on the presence of 

            # the ``-r`` option on the command line, this triggers a recover
            # the ``time.restart : True`` option in the da.rc file

        The latter is automatically set if the filter submits the next cycle at the end of the current one, 
        through method :meth:`~da.tools.initexit.CycleControl.submit_next_cycle`.

        The specific call tree under each scenario is: 

            1. *Fresh Start*
                *  :meth:`~da.tools.initexit.CycleControl.setup_file_structure()`  <- Create directory tree
            2. *Restart*
                *  :meth:`~da.tools.initexit.CycleControl.setup_file_structure()`
                *  :meth:`~da.tools.initexit.CycleControl.random_seed`    <- Read the random seed from file
            3. *Recover*
                *  :meth:`~da.tools.initexit.CycleControl.setup_file_structure()`
                *  :meth:`~da.tools.initexit.CycleControl.recover_run()`          <- Recover files from restart/one-ago dir, reset ``time.start``
                *  :meth:`~da.tools.initexit.CycleControl.random_seed` 

        And is always followed by a call to

            * parse_times()
            * WriteRc('jobfilename')
        """        
#
# case 1: A recover from a previous crash, this is signaled by flag "-r"
#
        if self['da.crash.recover']:
            logging.info("Recovering simulation from data in: %s" % self['dir.da_run'])
            self.setup_file_structure()
            self.recover_run()
            self.random_seed('read')
#
# case 2: A continuation, this is signaled by rc-item time.restart = True
#
        elif self['time.restart']:
            logging.info("Restarting filter from previous step")
            self.setup_file_structure()
            strippedname = os.path.split(self['jobrcfilename'])[-1]
            self['jobrcfilename'] = os.path.join(self['dir.exec'], strippedname)
            self.random_seed('read')
#
# case 3: A fresh start, this is signaled by rc-item time.restart = False
#
        elif not self['time.restart']:
            logging.info("First time step in filter sequence")
            self.setup_file_structure()

            # expand jobrcfilename to include exec dir from now on.
            # First strip current leading path from filename

            strippedname = os.path.split(self['jobrcfilename'])[-1]
            self['jobrcfilename'] = os.path.join(self['dir.exec'], strippedname)

        self.parse_times()
        self.write_rc(self['jobrcfilename'])

    def setup_file_structure(self):
        """ 
        Create file structure needed for data assimilation system.
        In principle this looks like:

            * ``${da_rundir}``
            * ``${da_rundir}/input``
            * ``${da_rundir}/output``
            * ``${da_rundir}/exec``
            * ``${da_rundir}/diagnostics``
            * ``${da_rundir}/analysis``
            * ``${da_rundir}/jobs``
            * ``${da_rundir}/restart/current``
            * ``${da_rundir}/restart/one-ago``

        .. note:: The exec dir will actually be a simlink to the directory where
                 the observation operator executable lives. This directory is passed through
                 the ``da.rc`` file. 

        .. note:: The observation input files will be placed in the exec dir,
                 and the resulting simulated values will be retrieved from there as well.

        """
        from da.tools.general import create_dirs

# Create the run directory for this DA job, including I/O structure

        filtertime = self['time.start'].strftime('%Y%m%d')

        self['dir.exec'] = os.path.join(self['dir.da_run'], 'exec')
        self['dir.input'] = os.path.join(self['dir.da_run'], 'input')
        self['dir.output'] = os.path.join(self['dir.da_run'], 'output', filtertime)
        self['dir.diagnostics'] = os.path.join(self['dir.da_run'], 'diagnostics')
        self['dir.analysis'] = os.path.join(self['dir.da_run'], 'analysis')
        self['dir.jobs'] = os.path.join(self['dir.da_run'], 'jobs')
        self['dir.restart'] = os.path.join(self['dir.da_run'], 'restart')
        self['dir.restart.current'] = os.path.join(self['dir.restart'], 'current')
        self['dir.restart.oneago'] = os.path.join(self['dir.restart'], 'one-ago')

        create_dirs(self['dir.da_run'])
        create_dirs(os.path.join(self['dir.exec']))
        create_dirs(os.path.join(self['dir.input']))
        create_dirs(os.path.join(self['dir.output']))
        create_dirs(os.path.join(self['dir.diagnostics']))
        create_dirs(os.path.join(self['dir.analysis']))
        create_dirs(os.path.join(self['dir.jobs']))
        create_dirs(os.path.join(self['dir.restart']))
        create_dirs(os.path.join(self['dir.restart.current']))
        create_dirs(os.path.join(self['dir.restart.oneago']))

        logging.info('Succesfully created the file structure for the assimilation job')
#LU tutaj chyba brakuje move restart data
    def recover_run(self):
        """
        Prepare a recovery from a crashed run. This consists of: 
        
            - copying all data from the restart/one-ago folder (:meth:`~da.tools.initexit.CycleControl.move_restart_data`),
            - replacing all ``rc-file`` items with those from the ``da_runtime.rc`` in the restart/current dir 
            - resetting the seed of the random number generator to the value it had before the crash (:meth:`~da.tools.initexit.CycleControl.random_seed`)
            - replacing the output dir name, since it has the sample time in it...
            
        """
        import da.tools.rc as rc
        from da.tools.general import create_dirs

        # Replace rc-items with those from the crashed run's last rc-file (now in restart.current dir)

        file_rc_rec = os.path.join(self['dir.restart.current'], 'da_runtime.rc')
        rc_rec = rc.read(file_rc_rec)

        for k, v in rc_rec.iteritems():
            self[k] = v

        self.validate_rc()

        logging.debug("Replaced rc-items.... ")
        logging.debug("Next cycle start date is %s" % self['time.start'])

        # Copy randomseed.pickle file to exec dir
        source = os.path.join(self['dir.restart.current'], 'randomseed.pickle') #LU wydaje mi sie ze tutaj nie trzeba podawac nazwy pliku w folderze docelowym, jesli sie obczai ze to folder to sie kopiuje.
        dest = os.path.join(self['dir.exec'], 'randomseed.pickle')
        shutil.copy(source, dest)

        logging.debug("Replaced randomseed file with previous cycles' last values")
        # Re-create the output dir for this time step, if needed

        filtertime = self['time.start'].strftime('%Y%m%d')
        self['dir.output'] = os.path.join(self['dir.da_run'], 'output', filtertime)
        create_dirs(os.path.join(self['dir.output']))

    def finalize(self):
        """
        finalize the da cycle, this means writing the save data and rc-files for the next run. 
        The following sequence of actions occur:

            * Write the randomseed to file for reuse in next cycle
            * Write a new ``rc-file`` with ``time.restart : True``, and new ``time.start`` and ``time.end``
            * Collect all needed data needed for check-pointing (restart from current system state)
            * Move the previous check pointing data out of the way, and replace with current
            * Submit the next cycle

        """
        self.random_seed('write')
        self.write_new_rc_file()
        self.move_restart_data(io_option='store')  # Move restart data from current to one-ago
        self.collect_restart_data()  # Collect restart data for next cycle into a clean restart/current folder
        self.collect_output()  # Collect restart data for next cycle into a clean restart/current folder
        self.submit_next_cycle()

    def collect_output(self):
        """ Collect files that are part of the requested output for this cycle. This function allows users to add files 
            to a list, and then the system will copy these to the current cycle's output directory.
            The list of files included is read from the 
            attribute "OutputFileList" which is a simple list of files that can be appended by other objects/methods that
            require output data to be saved.


        """
        from da.tools.general import create_dirs

        targetdir = os.path.join(self['dir.output'])
        create_dirs(os.path.join(targetdir))

        logging.info("Collecting the required output data") 
        logging.debug("           to   directory: %s " % targetdir)

        for file in set(self.OutputFileList):
            if os.path.isdir(file): # skip dirs
                continue
            if not os.path.exists(file): # skip dirs
                logging.warning("           [not found] .... %s " % file)
                continue

            logging.debug("           [copy] .... %s " % file)
            shutil.copy(file, file.replace(os.path.split(file)[0], targetdir))



    def collect_restart_data(self):
        """ Collect files needed for the restart of this cycle in case of a crash, or for the continuation of the next cycle. 
            All files needed are written to the restart/current directory. The list of files included is read from the 
            attribute "RestartFileList" which is a simple list of files that can be appended by other objects/methods that
            require restart data to be saved.

            .. note:: Before collecting the files in the ``RestartFileList``, the restart/current directory will be emptied and
                     recreated. This prevents files from accumulating in the restart/current and restart/one-ago folders. It 
                     also means that if a file is missing from the ``RestartFileList``, it will not be available for check-pointing
                     if your run crashes or dies!

            Currently, the following files are included:

                * The ``da_runtime.rc`` file
                * The ``randomseed.pickle`` file
                * The savestate.nc file
                * The files in the ``ObservationOperator.RestartFileList``, i.e., restart data for the transport model


            .. note:: We assume that the restart files for the :ref:`ObservationOperator` 
                      reside in a separate folder, i.e, the ObservationOperator does *not* write directly to the CTDAS restart dir!

        """
        from da.tools.general import create_dirs

        targetdir = os.path.join(self['dir.restart.current'])

        logging.info("Purging the current restart directory before collecting new data")

        create_dirs(os.path.join(targetdir), forceclean=True)

        logging.info("Collecting the required restart data")
        logging.debug("           to   directory: %s " % targetdir)

        for file in set(self.RestartFileList):
            if os.path.isdir(file): # skip dirs
                continue
            if not os.path.exists(file): # skip dirs
                logging.warning("           [not found] .... %s " % file)
            else:
                logging.debug("           [copy] .... %s " % file)
                shutil.copy(file, file.replace(os.path.split(file)[0], targetdir))


    def move_restart_data(self, io_option='restore'):
        """ 
        Store or restore model state to/from a restart directory. 

            Two IO options are available:

            (1) io_option = restore    : Get data from restart.oneago directory
            (2) io_option = store      : Save data to restart.oneago directory

            In case of a 'store' command the restart.oneago folder is re-created so that the contents are empty to begin with.

        """
        from da.tools.general import create_dirs

        if io_option not in ['store', 'restore']:
            raise ValueError, 'Invalid option specified for io_option (%s)' % io_option

        if io_option == 'store':
            targetdir = self['dir.restart.oneago']
            sourcedir = self['dir.restart.current']
        elif io_option == 'restore':
            sourcedir = self['dir.restart.oneago']
            targetdir = self['dir.restart.current']

# If "store" is requested, recreate target dir, cleaning the contents 

        if io_option == 'store':
            create_dirs(os.path.join(targetdir), forceclean=True)

        logging.debug("Performing a %s of data" % io_option)
        logging.debug("           from directory: %s " % sourcedir)
        logging.debug("           to   directory: %s " % targetdir)


        for file in os.listdir(sourcedir):
            file = os.path.join(sourcedir, file)
            if not os.path.exists(file):
                logging.debug("Cannot find requested file to move: %s " % file)  
                sys.exit(2)

            if os.path.isdir(file): # skip dirs
                logging.debug("           [skip] .... %s " % file)
                continue    
            else:
                logging.debug("           [copy] .... %s " % file)
                shutil.copy(file, file.replace(sourcedir, targetdir))

#
    def write_new_rc_file(self):
        """ Write the rc-file for the next DA cycle. 

            .. note:: The start time for the next cycle is the end time of this one, while 
                      the end time for the next cycle is the current end time + one cycle length. 
                      
            The resulting rc-file is written to the ``dir.exec`` so that it can be used when resubmitting the next cycle
            
        """
        import da.tools.rc as rc
        import copy

        # We make a copy of the current DaCycle object, and modify the start + end dates and restart value

        newDaCycle = copy.deepcopy(self)
        newDaCycle.advance_cycle_times()
        newDaCycle['time.restart'] = True

        # Create the name of the rc-file that will hold this new input, and write it

        fname = os.path.join(self['dir.exec'], 'da_runtime.rc')  # current exec dir holds next rc file
        rc.write(fname, newDaCycle)
        logging.debug('Wrote new da_runtime.rc (%s) to exec dir' % fname)

        # The rest is info needed for a system restart, so it modifies the current DaCycle object (self)

        self['da.restart.fname'] = fname    # needed for next job template
        self.RestartFileList.append(fname)  # current restart list holds next rc file name

        logging.debug('Added da_runtime.rc to the RestartFileList for later collection')


    def write_rc(self, fname):
        """ Write RC file after each process to reflect updated info """
        import da.tools.rc as rc

        rc.write(fname, self)
        logging.debug('Wrote expanded rc-file (%s)' % (fname))
        

    def submit_next_cycle(self):
        """ 
        Submit the next job of a DA cycle, this consists of 
            * Changing to the working directory from which the job was started initially
            * create a line to start the master script again with a newly created rc-file
            * Submitting the jobfile 

        If the end of the cycle series is reached, no new job is submitted.

        """
        from string import join

        if self['time.end'] < self['time.finish']:

            # file ID and names
            jobid = self['time.end'].strftime('%Y%m%d') 
            targetdir = os.path.join(self['dir.exec'])
            jobfile = os.path.join(targetdir, 'jb.%s.jb' % jobid)
            logfile = os.path.join(targetdir, 'jb.%s.log' % jobid)
#LU tutaj sa parametry ktore ida na gore do pliku job. nie zawsze koniecznie potrzebne.
            # Template and commands for job
            jobparams = {'jobname':"j.%s" % jobid, 'jobtime':'06:00:00', 'logfile': logfile, 'errfile': logfile}
            template = self.DaPlatForm.get_job_template(jobparams)
            execcommand = os.path.join(self['dir.da_submit'], sys.argv[0]) 
            template += 'python %s rc=%s %s >&%s' % (execcommand, self['da.restart.fname'], join(self.opts, ''),logfile) 

            # write and submit 
            self.DaPlatForm.write_job(jobfile, template, jobid)
            jobid = self.DaPlatForm.submit_job(jobfile, joblog=logfile) 
        else:
            logging.info('Final date reached, no new cycle started')


def start_logger(level=logging.INFO):
    """ start the logging of messages to screen"""

# start the logging basic configuration by setting up a log file

    logging.basicConfig(level=level,
                        format=' [%(levelname)-7s] (%(asctime)s) py-%(module)-20s : %(message)s',
                        datefmt='%Y-%m-%d %H:%M:%S')

def parse_options():
    """ 
    Function parses options from the command line and returns the arguments as a dictionary.
    Accepted command line arguments are:

    ========  =======
    Argument  Meaning
    ========  =======
    -v        verbose output in log files
    -h        display help
    -r        start a simulation by recovering from a previous crash
    ========  =======

    """
    import getopt

# Parse keywords, the only option accepted so far is the "-h" flag for help

    opts = []
    args = []
    try:                                
        opts, args = getopt.gnu_getopt(sys.argv[1:], "-rv")
    except getopt.GetoptError, msg:           
        logging.error('%s' % msg)
        sys.exit(2)      

    for options in opts:
        options = options[0].lower()
        if options == '-r':
            logging.info('-r flag specified on command line: recovering from crash')
        if options == '-v':
            logging.info('-v flag specified on command line: extra verbose output')
            logging.root.setLevel(logging.DEBUG)

    if opts: 
        optslist = [item[0] for item in opts] #LU ze co same minusy zwroci?
    else:
        optslist = []

# Parse arguments and return as dictionary

    arguments = {}
    for item in args:
        #item=item.lower()

# Catch arguments that are passed not in "key=value" format

        if '=' in item:
            key, arg = item.split('=')
        else:
            logging.error('%s' % 'Argument passed without description (%s)' % item)
            raise getopt.GetoptError, arg

        arguments[key] = arg


    return optslist, arguments

def validate_opts_args(opts, args):
    """ 
 Validate the options and arguments passed from the command line before starting the cycle. The validation consists of checking for the presence of an argument "rc", and the existence of
 the specified rc-file.  
 
    """
    if not args.has_key("rc"):
        msg = "There is no rc-file specified on the command line. Please use rc=yourfile.rc"
        logging.error(msg)
        raise IOError, msg
    elif not os.path.exists(args['rc']):
        msg = "The specified rc-file (%s) does not exist " % args['rc'] 
        logging.error(msg)
        raise IOError, msg

    # WP not needed anymore
    #if not args.has_key('process'):
    #    msg = "There is no process specified on the command line, assuming process=Start"   ; logging.info(msg)
    #    args['process'] = 'start'
    #if args['process'].lower() not in validprocesses:
    #    msg = "The specified process (%s) is not valid"%args['process']   ; logging.error(msg)
    #    raise IOError,msg

    return opts, args


if __name__ == "__main__":

    sys.path.append('../../')
    opts, args = parse_options()
    print opts
    print args

